require 'os'
require 'Bookmark'
require 'Utils'


# This class extracts bookmarks from a PDF file using a tool and converts that
# information into a readable HTML output
class PdfExtractor

  PDF_FILE = 'files/Dependency Injection with AngularJS - Knol, Alex.pdf'
  TEMPLATES = "templates"
  FILES = "files"
  
  # The initialization method
  def initialize( options )
    @options = options
  end
  

  include Utils
    
  # Extracts the bookmark info from PDF file ("from") and writes it into a txt file
  def extract( from )
    to = change_extension( result, '.txt' )
    if (OS.windows?)
      cmd_line = "\"#{pdf_tool()}\" \"#{from}\" dump_data output \"#{to}\""
    else
      cmd_line = "\"#{pdf_tool()}\" \"#{from}\" dump_data output - |tee \"#{to}\" >/dev/null"
    end
    if (! system( cmd_line ))
      raise SystemCallError.new('Could not invoke PDFtk tool')
    end    
    to
  end

  # Extracts the bookmarks from a text file yielding bookmark objects
  def each_bookmark( from )
    lines = File.open( from ).select { | line | line =~ /^Bookmark/ }
    title = 'Default'
    lines.each_index() do | idx |                                           # search the lines
      line = lines[idx]
      if( idx % 4 == 1 )                                                    # Title
        title = line[/^BookmarkTitle\: *(.*?$)/, 1]                         # extract from line
        title = strip_from_title( title )                                   # strip from the beginning
      end
      if( idx % 4 == 2 )                                                    # Level
        level = line[/^BookmarkLevel\: *(.*?$)/, 1].to_i
        bookmark = Bookmark.new( title, level )
        yield bookmark
      end
    end
  end
  
  # Instance method, parses a file into a Bookmark structure and returns the root node
  def parse( from )
    root = Bookmark.new( "Root" )
    cur_parent = root
    prev_child = nil
    level2_number = 0
    
    self.each_bookmark( from ) do | bookmark |
      
      if ( bookmark.level == cur_parent.level + 2 )               # prepare bookmark and its parent
        cur_parent = prev_child
      end
      while ( bookmark.level <= cur_parent.level )
        cur_parent = cur_parent.parent
      end
                                                                  # prepared !
      if (bookmark.level == 2)                                    # at this point we have level 2 bookmark 
        level2_number += 1                                        # increase counter
        if (cur_parent.children.length == 0)                      # it's the first child of parent
          cur_parent.number = level2_number                       # set its number in parent !
        end
      end
      
      bookmark.parent = cur_parent                                # finally add to tree
      cur_parent.children.push( bookmark )
      prev_child = bookmark

    end
    root
  end
    
end

